--[[
    This is a stab at making a menu system.  I didn't like the one that
    blizzard had for drop downs.  It could be seriously
    rewritten so that it was more generally useful and easier to follow.

    Thanks to blizz for the good reference point though.
--]]

local ZooFu = ZooKeeper:NewClass('Frame')
ZooKeeper.ZooFuFrame = ZooFu

TYPE_MOUNTS = "MOUNT"
TYPE_COMPANIONS = "CRITTER"

--local references
local _G = getfenv(0)
local L = LibStub('AceLocale-3.0'):GetLocale('ZooKeeper')

--frame constructor
local lastID = 1
local buttonID = 1
local sideFrameID = 1
local buttonHeightPadding = 2
local buttonWidthPadding = 5
local iconWidth = 16
local minButtonWidth = 80
local scrollWidth = 10
local textPadding = 5
local minButtonHeight = 15
local minTotalHeight = 30
local showTime = 1
local allButtons = {}
local allSideFrames = {}

function ZooFu:New(titleText, db, zooInstance)
    local f = self:Bind(CreateFrame('Frame', format('ZooFuFrame%d', lastID), UIParent, 'ZooFuMenuTemplate'))
    f.db = db
    f:CreateMenu()

    f:SetBackdropColor(0, 0, 0, 0.6)

    self.Zoo = zooInstance

    table.insert(UISpecialFrames, f:GetName())
    lastID = lastID + 1

    return f
end

function ZooFu:CreateMenu()
    self:CreateButtonData()
    self:CreateMenuButtons(UIParent, self, self.buttonList, true)

    self:SetScript("OnEnter", function(self) self:StopCounting(self) end);
    self:SetScript("OnLeave", function(self) self:StartCounting(self) end);
    self:SetScript("OnUpdate", function(self, elapsed) self:OnUpdate(self, elapsed) end);
end

function ZooFu:ShowMenu(parentFrame)
    self:SetPoint("TOPLEFT", parentFrame, "BOTTOMLEFT", 0, 2)
    if not self:IsShown() then
        self:Show()
    end
end

function ZooFu:OnClick(frame)
    local func = frame.func
    if func then
        func(self, unpack(frame.args))
        self:Hide()
        self:HideSides(self)
    end
end

function ZooFu:OnUpdate(self, elapsed)
    if ( not self.showTimer ) then
        return;
    elseif ( self.showTimer < 0 ) then
        self:Hide();
        self:HideSides(self);
        self.showTimer = nil;
    else
        self.showTimer = self.showTimer - elapsed;
    end
end

-- Start the countdown on a frame
function ZooFu:StartCounting(self)
    self.showTimer = showTime;
end

-- Stop the countdown on a frame
function ZooFu:StopCounting(self)
    self.showTimer = nil;
end

function ZooFu:HideSides(parent)
    local sidef
    for i = 1, getn(parent.sideFrames) do
        sidef = parent.sideFrames[i]
        sidef:Hide();
    end
end

function ZooFu:OnEnter(self, frame)
    self:StopCounting(self);

    if frame then
        self:HideSides(frame.parent);
        if frame.sideFrame then
            frame.sideFrame:Show();
        end
        getglobal(frame:GetName().."Highlight"):Show();
    end
end

function ZooFu:OnLeave(self, frame)
    self:StartCounting(self)

    if frame then
        getglobal(frame:GetName().."Highlight"):Hide();
    end
end

--[[
    Button logic
--]]

function ZooFu:CreateButtonData()
    self.buttonList = {}
    self.buttonList[1] = {
        name = L["CompanionMenuName"]
    }
    self.buttonList[2] = {
        name = L["MountMenuName"]
    }

    self.buttonList[1].list = self:FillInList(TYPE_COMPANIONS)
    self.buttonList[2].list = self:FillInList(TYPE_MOUNTS)
end

function ZooFu:FillInList(mode)
    local creatureID, creatureName, spellID, icon, active;
    local numPets = GetNumCompanions(mode)

    list = {}

    local names = {}
    local translator = {}
    for i = 1, numPets do
        creatureID, creatureName, spellID, icon, active = GetCompanionInfo(mode, i)
        table.insert(names, creatureName)
        translator[creatureName] = i
    end
    table.sort(names)

    local k = 0
    local offset = 0

    -- insert no pet first (if filling out companions)
    if mode == TYPE_COMPANIONS then
        offset = 1
        k = 1
        list[k] = {}
        list[k].name = L.NoCompanion
        list[k].func = function(self) self.Zoo:HideAndClearLastCompanion() end
        list[k].args = {}
    end

    -- then add others
    for i = 1, numPets do
        k = i + offset
        creatureName = names[i]
        creatureID = translator[creatureName]
        list[k] = {}
        list[k].name = creatureName
        list[k].func = function(self, mode, creatureID) self:CallCompanion(self, mode, creatureID) end
        list[k].args = {mode, creatureID}
    end

    return list
end

function ZooFu:CallCompanion(self, mode, creatureID)
    self.db.char.lastCompanion = creatureID
    CallCompanion(mode, creatureID)
end

function ZooFu:InitScroll(frame)
    if not frame.lastPos then
        frame.lastPos = 0
    end

    local offset = frame.lastPos;
    FauxScrollFrame_SetOffset(frame, offset)
    self:UpdateScroll(frame, offset)
end

function ZooFu:UpdateScroll(frame, offset)
    local i, index, item;
    local b, f, list, scrollSize, lastPos;

    scrollSize = self.db.profile.fuBarScrollHeight;
    list = frame.list

    if getn(frame.frameList) > 0 then
        FauxScrollFrame_Update(frame, getn(list), scrollSize, scrollSize - 1)
        lastPos = FauxScrollFrame_GetOffset(frame)
        frame.lastPos = lastPos

        for i = 1, getn(list) do
           list[i].frame = nil
        end

        for i = 1, scrollSize do
            index = i + lastPos;
            f = frame.frameList[i]
            if index <= getn(list) then
                b = list[index]
                b.frame = f
                self:SetupButton(b)
                f:Show()
            elseif f ~= nil then
                f:Hide()
            end
        end
    end
end

function ZooFu:CreateMenuButtons(parentFrame, currentFrame, currentList, calcHeight)
    local b, last, normalText
    local width, height
    local numItems
    local totalWidth = 1
    local totalHeight = 0

    currentFrame.sideFrames = {}

    if not calcHeight and self.db.profile.fuBarScrollHeight ~= nil  then
        numItems = self.db.profile.fuBarScrollHeight
        if getn(currentList) < numItems then
            numItems = getn(currentList)
        end
    else
        numItems = getn(currentList)
    end

    currentFrame.list = currentList
    currentFrame.frameList = {}
 
    for i = 1, numItems do
        b = currentList[i]
        b.relativeTo = last

        if not b.frame then
            self:CreateButton(b, currentFrame)
            currentFrame.frameList[i] = b.frame
        end

        if b.list and getn(b.list) > 0 then
            self:CreateSideMenu(b)
            self:CreateMenuButtons(currentFrame, b.frame.sideFrame, b.list, false)
        end

        -- get width
        width = b.frame:GetTextWidth()
        if width < minButtonWidth then
            width = minButtonWidth
        end
        width = width + (2 * buttonWidthPadding) + iconWidth
        -- adjust totals
        if width > totalWidth then
            totalWidth = width
        end

        -- get height
        height = b.frame:GetTextHeight()
        if height < minButtonHeight then
            height = minButtonHeight
        end
        height = height + buttonHeightPadding
        totalHeight = totalHeight + height + buttonHeightPadding

        b.frame:SetWidth(width)
        b.frame:SetHeight(height)

        last = b
    end

    if getn(currentFrame.frameList) > 0 then
       currentFrame.sampleHeight = currentFrame.frameList[1]:GetHeight()
    end

    for i = 1, getn(currentList) do
        b = currentList[i]
        if b.frame then
           b.frame:SetWidth(totalWidth)
        end
    end

    -- Set up script for scrollbar
    if not calcHeight and self.db.profile.fuBarScrollHeight ~= nil then
       currentFrame:SetScript("OnVerticalScroll", 
           function(self, offset) FauxScrollFrame_OnVerticalScroll(
               self, offset, self.sampleHeight, 
                   function(self, offset) 
                       self.top:UpdateScroll(self, offset) 
                   end) 
           end);
        self:InitScroll(currentFrame);
    end

    totalHeight = totalHeight + buttonHeightPadding;
    if totalHeight < minTotalHeight then
        totalHeight = minTotalHeight
    end

    currentFrame:SetHeight(totalHeight);
    currentFrame:SetWidth(totalWidth);

end

function ZooFu:CreateSideMenu(b)
    local prevSideFrame = allSideFrames[b.name]
    if prevSideFrame and prevSideFrame ~= nil then
        -- repurpose a previously created frame
        b.frame.sideFrame = prevSideFrame
        b.frame.sideFrame:SetParent(b.frame.parent)
    else
        b.frame.sideFrame = CreateFrame("ScrollFrame", format('ZooFuSideFrame%d', sideFrameID), b.frame.parent, 'ZooFuScrollMenuTemplate');
        allSideFrames[b.name] = b.frame.sideFrame;
        b.frame.sideFrame.sideFrameID = sideFrameID
        sideFrameID = sideFrameID + 1
    end

    b.frame.sideFrame.top = self;
    b.frame.sideFrame:ClearAllPoints()
    b.frame.sideFrame:SetPoint("TOPLEFT", b.frame, "TOPRIGHT", -5, 0)
    b.frame.sideFrame:SetScript("OnEnter", function(self) self.top:OnEnter(self.top); end)
    b.frame.sideFrame:SetScript("OnLeave", function(self) self.top:OnLeave(self.top); end)
    b.frame.sideFrame:SetBackdropColor(0, 0, 0, 0.6)

    if getn(b.list) > 0 then
        local arrowFrame = getglobal(b.frame:GetName().."ExpandArrow")
        arrowFrame:Show();
    end

    b.frame.parent.sideFrames[b.frame.sideFrame.sideFrameID] = b.frame.sideFrame;

end

function ZooFu:CreateButton(b, parent)
    local prevButtonFrame = allButtons[b.name]
    if prevButtonFrame and prevButtonFrame ~= nil then
        -- repurpose a previously created button
        b.frame = prevButtonFrame
        b.frame:SetParent(parent)

        -- reset state
        local arrowFrame = getglobal(b.frame:GetName().."ExpandArrow")
        arrowFrame:Hide();
        b.frame.sideFrame = nil
    else
        b.frame = CreateFrame("Button", format('ZooFuMenuButton%d', buttonID), parent, 'ZooFuMenuButtonTemplate');
        allButtons[b.name] = b.frame
        b.frame.buttonID = buttonID
        buttonID = buttonID + 1
    end

    b.frame.top = self
    b.frame.parent = parent

    if b.relativeTo then
        b.frame:SetPoint("TOPLEFT", b.relativeTo.frame, "BOTTOMLEFT", 0, (-1 * buttonHeightPadding))
    else
        b.frame:SetPoint("TOPLEFT", parent, "TOPLEFT", buttonWidthPadding, (-1 * buttonHeightPadding))
    end

    self:SetupButton(b)

    local checkmark = getglobal(b.frame:GetName().."Check");
    if b.checkmark then
        checkmark:Show();
    else
        checkmark:Hide();
    end
end

function ZooFu:SetupButton(b)
    local normalText = getglobal(b.frame:GetName().."NormalText");

    b.buttonID = b.frame.buttonID
    b.frame.func = b.func
    b.frame.args = b.args

    normalText:ClearAllPoints()
    normalText:SetPoint("LEFT", b.frame, "LEFT", textPadding, 0);
    normalText:SetText("|cFFFFFFFF"..b.name.."|r")

    b.frame:SetScript("OnEnter", function(self) self.top:OnEnter(self.top, self) end)
    b.frame:SetScript("OnLeave", function(self) self.top:OnLeave(self.top, self) end)

    if b.frame.func then
        if not b.frame.args then
            b.frame.args = {}
        end
        b.frame:Enable();
        b.frame:SetScript("OnClick", function(self) self.top:OnClick(self) end);
    end
end
