
--[[
    Future posibilities:  See ticket log

    Thanks to Kaelten (OneBag), pastamancer (Grid), Tuller (Combuctor) and ckknight (tons!)
        for their excellent, clean code to work from as reference points.
--]]

ZOOKEEPER_VERSION = "2.0"
TYPE_MOUNTS = "MOUNT"
TYPE_COMPANIONS = "CRITTER"

ZooKeeper = LibStub("AceAddon-3.0"):NewAddon("ZooKeeper", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale("ZooKeeper", true)
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")

-- BINDINGS
BINDING_HEADER_ZOOKEEPER = L.BindingHeader
BINDING_NAME_ZOOKEEPER_RECALL_HOTKEY = L.RecallHotkey
BINDING_NAME_ZOOKEEPER_NO_COMPANION = L.NoCompanionHotkey

function ZooKeeper:OnInitialize()
    -- persist db
    local baseDB = LibStub("AceDB-3.0"):New("ZooKeeperDB", nil, "Defaults")
    self.db = baseDB
    self:InitDB()

    -- options
    self:SetupOptions()

end

function ZooKeeper:InitDB()
    local version = self.db.char.version
    if not version or version ~= ZOOKEEPER_VERSION then
        self.db.char.version = ZOOKEEPER_VERSION
    end
    if self.db.global.version == 1.0 then
        -- in version 1.0, everything was global (bleck)
        -- copy from global to profile default or char
        self.db:SetProfile("Defaults")
        self.db.char.companionOrder = self.db.global.companionOrder
        self.db.char.lastCompanion = self.db.global.lastCompanion
        self.db.char.lastViewed = self.db.global.lastViewed
        self.db.char.mountOrder = self.db.global.mountOrder
        self.db.char.position = self.db.global.position
        self.db.char.searchFilter = self.db.global.searchFilter
        self.db.char.version = self.db.global.version
        self.db.profile.buttonPadding = self.db.global.buttonPadding
        self.db.profile.columns = self.db.global.columns
        self.db.profile.opacity = self.db.global.opacity
        self.db.profile.position = self.db.global.position
        self.db.profile.recallOnLanding = self.db.global.recallOnLanding
        self.db.profile.recallOnRez = self.db.global.recallOnRez
        self.db.profile.recallOnZone = self.db.global.recallOnZone
        self.db.profile.scale = self.db.global.scale
        self.db.global.version = nil
    end

    if not self.db.profile.columns then
        self.db.profile.columns = 6
    end
    if not self.db.profile.rows then
        self.db.profile.rows = 10
    end
    if not self.db.char.lastViewed then
        self.db.char.lastViewed = TYPE_MOUNTS
    end
    if not self.db.profile.buttonPadding then
        self.db.profile.buttonPadding = 4
    end
    if not self.db.profile.scale then
        self.db.profile.scale = 0.8
    end
    if not self.db.profile.opacity then
        self.db.profile.opacity = 0.7
    end
    if not self.db.char.mountOrder then
        self.db.char.mountOrder = {}
    end
    if not self.db.char.companionOrder then
        self.db.char.companionOrder = {}
    end
    if not self.db.profile.fuBarScrollHeight then
        self.db.profile.fuBarScrollHeight = 20
    end

    -- nil is valid for these:
    --     self.db.char.searchFilter
    --     self.db.char.lastCompanion
    --     self.db.char.position
    --     self.db.profile.position

    -- default recall options to true
    if self.db.profile.recallOnLanding == nil then
        self.db.profile.recallOnLanding = true
    end
    if self.db.profile.recallOnRez == nil then
        self.db.profile.recallOnRez = true
    end
    if self.db.profile.recallOnZone == nil then
        self.db.profile.recallOnZone = true
    end
end


function ZooKeeper:OnEnable()
    -- Called when the addon is enabled
    if not self.frame then
        self.frame = self.Frame:New(L.Title, self.db, 'pets');
    end

    self:CreateLDB();

    self.onTaxi = nil

    -- register stuffs
    self:RegisterEvent("COMPANION_UPDATE", "CompanionUpdateHandler");
    self:RegisterEvent("COMPANION_LEARNED", "CompanionLearnedHandler");
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "EnteringWorldHandler");
    self:RegisterEvent("PLAYER_UNGHOST", "UnghostHandler");
    self:ScheduleRepeatingTimer(self.OnUpdate, 2, self)
end

function ZooKeeper:CreateLDB()
    self.zooldb = ldb:NewDataObject("ZooKeeper", {
        type = "launcher",
	text = "Zoo",
        icon = "Interface\\Addons\\ZooKeeper\\zk_icon.tga",
        OnEnter = function(parentFrame)
            self:OnLDBEnter(parentFrame)
        end,
        OnLeave = function(parentFrame)
            self:OnLDBLeave()
        end,
        OnClick = function(parentFrame, button)
            if(button == "LeftButton") then
                self:OnLDBClick()
            else
	        self:OnLDBRightClick()
            end
        end,
     })
end

function ZooKeeper:OnDisable()
    -- unregister
    self:UnregisterAllEvents();

    -- cancel timers
    self:CancelAllTimers();

    -- Unhook character frame
    self:Unhook("CharacterFrameTab_OnClick");

    -- hide frames
    self.frame:Hide()
    if zooFuFrame then
       self.zooFuFrame:Hide()
    end
end

function ZooKeeper:CompanionUpdateHandler()
    if self.frame then
        self.frame:UpdatePets()
    end
end

function ZooKeeper:CompanionLearnedHandler()
    self:Update()
    local optionsFrame = self.zooFuFrame
    if optionsFrame then
       optionsFrame:CreateMenu()
    end
end

function ZooKeeper:Update()
    if self.frame and self.frame:IsShown() then
        self.frame:UpdatePets()
    end
end

function ZooKeeper:ShowZoo()
    self.frame:ShowFrame()
end

function ZooKeeper:ToggleZoo()
    if self.frame:IsShown() then
        self.frame:Hide()
    else
        self.frame:Show()
    end
end

function ZooKeeper:UnghostHandler()
    if self.db.profile.recallOnRez then
        self:RecallLastCompanion()
    end
end

-- Called on PLAYER_ENTERING_WORLD (anytime a load screen is shown)
function ZooKeeper:EnteringWorldHandler()
    -- let's check if we're flying so we don't zone in falling
    local isMounted = IsMounted()
    if not self.onMount and isMounted then
       -- set the mounted flag if we're really mounted (maybe we just logged in)
        self.onMount = true
    end

    if self.db.profile.recallOnZone and not isMounted then
        self:RecallLastCompanion()
    end
end

function ZooKeeper:RecallLastCompanion()
    local petID = self.db.char.lastCompanion
    if petID then
        local creatureID, creatureName, spellID, icon, active = GetCompanionInfo(TYPE_COMPANIONS, petID);
        if (not active) then
            CallCompanion(TYPE_COMPANIONS, petID)
         end
    end
end

function ZooKeeper:HideAndClearLastCompanion()
    DismissCompanion(TYPE_COMPANIONS)
    self.db.char.lastCompanion = nil
end


-- Taxi catcher, should rewrite some other way but PLAYER_LOST_CONTROL and PLAYER_GAINED_CONTROL doesn't seem to work.
--    Could instead only set the OnUpdate timer if recallOnLanding flag is set
function ZooKeeper:OnUpdate()
    if self.db.profile.recallOnLanding then
        local newOnTaxi = UnitOnTaxi("player")
        if self.onTaxi and not newOnTaxi then
            -- flight ended
            self:RecallLastCompanion()
        end

        self.onTaxi = newOnTaxi
    end

    if self.db.profile.recallOnDismount then
        local newOnMount = IsMounted()
        if self.onMount and not newOnMount then
            self:RecallLastCompanion()
        end

        self.onMount = newOnMount
    end
end

--[[
    LDB plugin
--]]

function ZooKeeper:OnLDBClick()
    self:ToggleZoo()
end

function ZooKeeper:OnLDBRightClick()
    InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end

function ZooKeeper:OnLDBEnter(parentFrame)
    if not self.zooFuFrame then
        self.zooFuFrame = self.ZooFuFrame:New(L.Title, self.db, self);
        self.zooFuFrame:Show()
    end

    self.zooFuFrame:ShowMenu(parentFrame)
end

function ZooKeeper:OnLDBLeave()
    local optionsFrame = self.zooFuFrame
    optionsFrame:StartCounting(optionsFrame)
end

--utility function: create a widget class (from Combuctor)
function ZooKeeper:NewClass(type)
    local class = CreateFrame(type)
    class.mt = {__index = class}

    function class:Bind(o)
        return setmetatable(o, self.mt)
    end

    return class
end
